#include "xttcps.h"

XTtcPs_Config XTtcPs_ConfigTable[] __attribute__ ((section (".drvcfg_sec"))) = {

	{
		"cdns,ttc", /* compatible */
		0xff0e0000, /* reg */
		0x8ab7547, /* xlnx,clock-freq */
		{0x4025}, /* interrupts */
		0xf9000000 /* interrupt-parent */
	},
	{
		"cdns,ttc", /* compatible */
		0xff0f0000, /* reg */
		0x8ab7547, /* xlnx,clock-freq */
		{0x4028}, /* interrupts */
		0xf9000000 /* interrupt-parent */
	},
	{
		"cdns,ttc", /* compatible */
		0xff100000, /* reg */
		0x8ab7547, /* xlnx,clock-freq */
		{0x402b}, /* interrupts */
		0xf9000000 /* interrupt-parent */
	},
	{
		"cdns,ttc", /* compatible */
		0xff110000, /* reg */
		0x8ab7547, /* xlnx,clock-freq */
		{0x402e}, /* interrupts */
		0xf9000000 /* interrupt-parent */
	},
	 {
		 NULL
	}
};